Content Insights React Native Integration
===========================

Android SDK setup
-----------------------

In order to use ContentInsights Android SDK, it is necessary that you copy provided `ContentInsightsSdk-x.x.x.aar` file into `<project_root>/android/app/libs` folder. 
In case `libs` folder does not exist, create it.


### build.gradle modification

In order for Gradle to include aar file in your project, it is necessary to modify <b>build.gradle</b> file at `<project_root>/android/app/build.gradle`

Under <b>dependencies</b> section in build.gradle file update:

From
`implementation fileTree(dir: "libs", include: ["*.jar"])`

To
`implementation fileTree(dir: "libs", include: ["*.jar", "*.aar"])`

In case above line does not exist, add it.

### Application code

In order to use ContentInsights SDK, it is required to create an Native module which is derived from `ReactContextBaseJavaModule` class. Native module class needs to implement getName method which represents the name of the module in ReactNative app.

```java
    /**
     * Get name of the module, to be used in ReactNative as NativeModules.{module_name}
     */
    @Override
    @NonNull
    public String getName() {
        return "ContentInsightsSDKModule";
    }
```

See: ContentInsightsSDKModule.java at `<project_root>/android/app/src/main/java/com.contentinsightssdkintegration`

Created module needs to be included in the Package class. 
It is necessary that created Native Package class implements  `ReactPackage` interface and that created Native module is added to package's module list. 
For example:

```java   
    /**
     * {@inheritDoc}
     */
    @Override
    @NonNull
    public List<NativeModule> createNativeModules(
            @NonNull ReactApplicationContext reactContext) {
        List<NativeModule> modules = new ArrayList<>();
        modules.add(new ContentInsightsSDKModule(reactContext, domainId));

        return modules;
    }
```

For more information, see ContentInsightsExamplePackage.java at `<project_root>/android/app/src/main/java/com.contentinsightssdkintegration`


iOS SDK setup
-----------------------

In order to use ContentInsights iOS SDK, it is necessary that you copy provided `ContentInsightsSdk-x.x.x.xcframework` folder into `<project_root>/ios` folder and to include it in your project.

### Podfile
If you are using CocoaPods, it is needed to add ```use_frameworks!``` to the Podfile in your Xcode project directory:

```
use_frameworks!
```

In case that above line causes a build error, it is needed to add it in the following way:

```
use_frameworks! :linkage => :static
```

### Application code

In order to create iOS Native module and have it exposes to React Native, it is needed that have the following components:

* Native Module implementation
* Bridging header
* Bridge

##### Native module

Native module file can be created in both Swift and Objective-C. 
It requires the following methods to be implemented:

* startArticleReadTracker
* startAttentionTracker
* startAttention
* pauseAttention
* setNewScrollPosition
* flushArticle

You can see and use example Native module swift class from our example application ```ContentInsightsModule.swift```

##### Bridging header 

This file is header file and it has to include following:

```
#import "RCTBridgeModule.h"
```

##### Bridge 

Bridge file contains code for exposing module and methods to ReactNative. You can see and m file  ```ContentInsightsBridge.m``` from our example application.

```
#import <Foundation/Foundation.h>
#import <RCTBridgeModule.h>

@interface RCT_EXTERN_MODULE(ContentInsightsModule, NSObject)

RCT_EXTERN_METHOD(startArticleReadTracker: (NSString *) referrerUrl articleUrl: (NSString *) articleUrl postId: (NSString *) postId)
RCT_EXTERN_METHOD(startAttentionTracker: (NSString *) referrerUrl articleUrl: (NSString *) articleUrl postId: (NSString *) postId)
RCT_EXTERN_METHOD(setScrollPosition: (NSInteger) position)
RCT_EXTERN_METHOD(startAttention)
RCT_EXTERN_METHOD(pauseAttention)
RCT_EXTERN_METHOD(flushArticle)

@end
```

React Native usage
-----------------------

## Tracking Article Load

Loading article means that article page/screen has been open to be read.

To track this event set desired parameters and request tracking to be done by the SDK:

```
NativeModules.ContentInsightsSDKModule.startArticleReadTracker("https://test-domain.com/article/test", "1", "test-domain.com");
```

## Tracking Article Read

Tracking Article Read will be done automatically by the SDK, 10s after ```startArticleReadTracker``` has been called.

## Tracking Attention Time

Attention time tracking needs to be initiated after ```startArticleReadTracker``` call.
To track this event set desired parameters and request tracking to be done by the SDK.

```
NativeModules.ContentInsightsSDKModule.startAttentionTracker("https://test-domain.com/article/test", "1", "test-domain.com");
```

In order to achieve maximum flexibility, tracking when the app/screen has user's attention is done manually.

Communication regarding when app gets or loses user's attention is done by calling the following methods: 

```
NativeModules.ContentInsightsSDKModule.startAttention()
``` 
or 

```
NativeModules.ContentInsightsSDKModule.pauseAttention()
```

Apart from notifying when the article has user attention or not, no further calls are required in order to track the attention time.

## Tracking Scrolling Activity

As part of tracking attention time, there is an option to track how much user has been scrolled.

In order to have valid statistics, there is a formula for calculating scrolling:

```
(viewportHeight + scrollTop) * 100 / scrollHeight
```

New scroll position is marked manually by calling ```setNewScrollPosition(newPos)```
whenever the scroll event occurs:

```
NativeModules.ContentInsightsSDKModule.setNewScrollPosition(newPos)
```

## Flushing the Article

When a user leaves the article, the tracker must flush the article. 
This is a sign for the tracker to finish the tracking of attention time and to do any final calls to ingestion servers.
When everything is finished, the tracker should be destroyed. 

To flush the article, please call the following method:

```
NativeModules.ContentInsightsSDKModule.flushArticle()
```
